from parser.parser_base import ParserBase


class FeatureEngineeringParser(ParserBase):

    def parse(self, data: list):
        return self._parse_feature_engineering_configs(data)

    def _parse_feature_engineering_configs(self, data: list):
        if (not data):
            return

        configs = []
        columns_set_alias = []
        for inputs in data:
            input = self._try_get(inputs, 'input')

            # columns
            columns_set, columns_alias = self._get_dataframe(
                self._try_get(input, 'columns'))
            columns_set_alias = columns_alias + columns_set_alias

            # features
            word_embedding, keyboard_smash = self._get_features_details(
                self._try_get(input, 'features'))
            data_lang, dimensions = self._get_word_embedding_config(
                word_embedding, columns_set_alias)

            # Enabled features
            enabled_features = keyboard_smash
            if (not dimensions):
                enabled_features['word_embedding'] = False
            else:
                enabled_features['word_embedding'] = True

            configs.append({
                'columns_alias': columns_alias,
                'columns_set': columns_set,
                'data_lang': data_lang,
                'dimensions': dimensions,
                'enabled_features': enabled_features
            })

        return configs, columns_set_alias

    def _get_dataframe(self, columns: dict):
        if (not columns):
            return

        columns_alias = []

        for column in columns:
            for key in column.keys():
                columns_alias.append(key)

        return columns, columns_alias

    def _get_features_details(self, features: dict):
        if (not features):
            return

        word_embedding = self._try_get(features, 'word_embedding')
        if (word_embedding == 'off'):
            word_embedding = False

        keyboard_smash = self._get_keyboard_smash_config(features)

        return word_embedding, keyboard_smash

    def _get_word_embedding_config(self, feature: dict, columns_alias: list):
        if (not feature):
            return 'es', None

        data_lang = self._get(feature, 'data_lang', 'es')
        if ('data_lang' in feature):
            del feature['data_lang']

        dimensions = {}
        dimensions_default_value = 25
        for key, item in feature.items():
            if (not (key in columns_alias)):
                error_msg = f'Label {key} not match'
                raise ValueError(error_msg)

            dimensions[key] = self._get(
                item, 'dimensions', dimensions_default_value)

        for name in columns_alias:
            if (not (name in dimensions)):
                dimensions[name] = dimensions_default_value

        return data_lang, dimensions

    def _get_keyboard_smash_config(self, features: dict):
        keyboard_smash_default_value = {
            'ksmash_sequence_vowels': True,
            'ksmash_sequence_consonants': True,
            'ksmash_sequence_special_characters': True,
            'ksmash_numbers': True,
            'ksmash_char_frequence': True
        }

        keyboard_smash = self._get(
            features, 'keyboard_smash', keyboard_smash_default_value)
        if (keyboard_smash == keyboard_smash_default_value):
            return keyboard_smash

        for key in keyboard_smash.keys():
            if (key in keyboard_smash_default_value
                    and keyboard_smash[key] == 'off'):
                keyboard_smash_default_value[key] = False

        return keyboard_smash_default_value



import os

from parser.YAML_parser import YAMLParser
from parser.feature_engineering_parser import FeatureEngineeringParser
from parser.model_parser import ModelParser

if __name__ != "__main__":
    exit()


def get_config():
    initialParser = YAMLParser
    featureEngineringParser = FeatureEngineeringParser
    modelParser = ModelParser

    for file in os.listdir('src/yamls'):
        filepath = os.path.join('src/yamls', file)
        config = initialParser(filepath).parse()

        features_configs, columns_set_alias = featureEngineringParser(
            filepath).parse(config['feature_engineering'])
        del config['feature_engineering']

        model_configs = modelParser(columns_set_alias).parse(config['model'])
        del config['model']

        print("FEATURES")
        print(features_configs)
        print(3*'\n')
        print(20*'-')
        print(3*'\n')
        print(model_configs)


get_config()



from statistics import mean


class KeySmash:
    """A class for calculating metrics to indicate key smashing behavior in a text.

    Key smashing is the act of typing on a keyboard in a rapid and uncontrolled manner,
    often resulting in a series of random characters being entered into a document or text field.
    """

    def __init__(self):
        self.char_sets = {
            "vowels": 'aeiouáéíóúãõ',
            "consonants": 'bcdfghjklmnñpqrstvwxyz',
            "special_characters": '!@#$%^¨|\'\"&*()_+:;~`´]}{[}ºª=-.¿¡'
        }

    def calculate_char_frequency_metric(self, text):
        """
        Calculate the Char Frequency Metric.

        Parameters
        ----------
        text : str
            The text to use for the calculation.

        Returns
        -------
        float
            Char Frequency Metric.

        Examples
        --------
        >>> calculate_char_frequency_metric("PUENTECILLA KM. 1.7")
        1.121212121212121
        >>> calculate_char_frequency_metric("ASDASD XXXX")
        3.0
        """
        word_results = []

        for w in text.split(' '):
            char_count = []
            if w and len(w) > 0:
                for e in set(w):
                    char_count.append(w.count(e)**2)
                word_results.append(sum(char_count)/len(w))

        if word_results == 0 or len(word_results) == 0:
            return 0
        else:
            return mean(word_results)

    def calculate_irregular_sequence_metric(self, text, opt):
        """
        Calculate the Irregular Sequence Metric.

        Parameters
        ----------
        text : str
            The text to use for the calculation.
        opt : str
            The type of characters to consider for the calculation,
            can be one of 'vowels', 'consonants', or 'special_characters'.

        Returns
        -------
        float
            Irregular Sequence Metric.

        Examples
        --------
        >>> calculate_irregular_sequence_metric("PUENTECILLA KM. 1.7", "vowels")
        0.21052631578947367
        >>> calculate_irregular_sequence_metric("ASDASD XXXX", "consonants")
        2.1818181818181817
        >>> calculate_irregular_sequence_metric("!@#$% ASDFGHJKL", "special_characters")
        1.5625
        """
        count_sequence = 1
        sequence_regex = []

        text = str(text).lower()
        opt = self.char_sets[opt]

        for i in range(len(text) - 1):
            if text[i] in opt and text[i + 1] in opt:
                count_sequence = count_sequence + 1
            else:
                if (count_sequence != 1):
                    sequence_regex.append(count_sequence**2)
                    count_sequence = 1

        if (count_sequence != 1):
            sequence_regex.append(count_sequence**2)

        return sum(sequence_regex)/len(text)

    def calculate_number_count_metric(self, text):
        """
        Calculate the Number Count Metric.

        Parameters
        ----------
        text : str
            The text field to use for the calculation.

        Returns
        -------
        float
            Number Count Metric.

        Examples
        --------
        >>> calculate_number_count_metric("ABC 123 !@#")
        0.0
        >>> calculate_number_count_metric("ABC123 !@#")
        0.9
        """
        text_list = text.split()
        calc_num_line = 0

        if text_list:
            for word in text_list:
                if any(char.isdigit() for char in word) and any(not char.isdigit() for char in word):
                    num = len([char for char in word if char.isdigit()])
                    calc_num = num**2
                    calc_num_line += calc_num

            return calc_num_line / len(' '.join(text_list))
        return 0



from pandas import DataFrame


def get_concatenated_column(csv, columns, column_name):
    if (type(columns) == str):
        csv[column_name] = csv[columns]
        return DataFrame(csv[column_name])

    for value in columns:
        if (not (column_name in csv)):
            csv[column_name] = csv[value].astype(str)
            continue

        csv[column_name] = csv[column_name] + ' ' + csv[value].astype(str)

    return DataFrame(csv[column_name])



from whatlies.language import BytePairLanguage
# import numpy as np


class WordEmbedding:
    def __init__(self, lang, dimensions):
        """Initialize a word embedding object using the BytePairLanguage model.

        Parameters
        ----------
        lang: str
            The language to use for the word embedding (default is "es")
        dimensions: str
            The number of dimensions for the word embedding vectors (default is 25)
        """
        self.bpl = BytePairLanguage(lang=lang, dim=dimensions)

    def get_embedding(self, text):
        """Get the word embedding vector for a given text.

        Parameters
        ----------
        text: str
            The string to be processed

        Returns
        ----------
        arr[float]
            A word embedding vector for the given text

        Examples
        --------
        >>> get_embedding("This is a text", "es", 25)
        [ 0.06799883  0.17547965  0.47599664  0.16108984 -0.1360625  -0.10632467
        -0.10654568 -0.09805    -0.33004168 -0.33528003 -0.23304085  0.36661038
        -0.5797167   0.53252834  0.30276018 -0.01584417  0.85087484  0.14121284
        0.74862367 -0.33011952  0.015432    0.02694534  0.10118082 -0.34017918
        -0.14560167]
        """
        return self.bpl[text].vector
